import anthropic
import csv

#Set up API key, Grok works with OpenAI and Anthropic SDKs, using Anthropic as it lets us set top_k and thus more reproducible outputs (in theory)
client = anthropic.Anthropic(
    api_key=YOUR_API_KEY,
    base_url="https://api.x.ai",
)

#Select type of causal genes 'New' or '' or 'Hard'
CausalGenes = 'New'

#read system prompt
with open('../SystemPrompts/SystemPromptInitial.txt') as file:
    SystemPrompt = file.read()

#Read list of tuples (Causal gene, Phenotype, [(Candidate gene ID, Candidate gene name, candidate gene symbol), (etc), (etc)])
with open(f'../../QTLGenesListMaker/data/{CausalGenes}PromptData50.tsv', newline='') as f:
    reader = csv.reader(f, delimiter='\t')
    PromptData = [tuple(row) for row in reader]

#Create list to store replies
GrokAnswers = []

#Iterate over all prompt info to generate 60 prompts
for index in range(0,len(PromptData)):
    #Create user prompt from causal gene, phenotype and candidate gene data and invariable text
    UserPrompt = "Identify the causal gene. QTL phenotype: {Phenotype}. Genes in locus: {CandidateGenes}".format(Phenotype = PromptData[index][1], CandidateGenes = PromptData[index][2])
    #Remove ' as it adds unnecessary chars (not necessary per se as this is a free API, but doesn't hurt)
    UserPrompt = UserPrompt.replace("'", "")

    # set temp = 0, top_k = 1 and top_p to a small non 0 value to achieve reproducible outputs, select Grok-2 Beta as the model
    response = client.messages.create(
        model="grok-beta",
        top_k=1,
        top_p=0.01,
        temperature=0,
        system= SystemPrompt,
        messages=[
            {"role": "user", "content": UserPrompt}
        ],
        max_tokens=1024
    )
    
    #Create a tuple (real causal gene ID, prediction output) and append to answers list, link each answer to the correct causal gene it belongs to
    #Output is messy, so take only the JSON part from it, find start and end first
    StartJSON = response.content[0].text.find('{')
    EndJSON = response.content[0].text.find('}')
    StartOfID = PromptData[index][0].find('AT')
    TempAnswerTuple = (PromptData[index][0][StartOfID:StartOfID+len("AT1G12345")], PromptData[index][1], response.content[0].text[StartJSON:EndJSON+1])
    GrokAnswers.append(TempAnswerTuple)

    #Counter for progress visibility
    print(index)

    #Write all the answers to a file
    with open(f'../Predictions/{CausalGenes}ModelComparison/Grok-2Beta_Predictions.tsv', mode="w", newline="") as file:
        writer = csv.writer(file, delimiter='\t')
        writer.writerows(GrokAnswers)